@@ -1,9 +1,9 @@
 # File	  : Zlib.pm
 # Author  : Paul Marquess
-# Created : 6th January 2000
-# Version : 1.08
+# Created : 15th January 2001
+# Version : 1.09
 #
-#     Copyright (c) 1995-2000 Paul Marquess. All rights reserved.
+#     Copyright (c) 1995-2001 Paul Marquess. All rights reserved.
 #     This program is free software; you can redistribute it and/or
 #     modify it under the same terms as Perl itself.
 #
@@ -21,7 +21,7 @@ use strict ;
 use vars qw($VERSION @ISA @EXPORT $AUTOLOAD 
 	    $deflateDefault $deflateParamsDefault $inflateDefault) ;
 
-$VERSION = "1.08" ;
+$VERSION = "1.09" ;
 
 @ISA = qw(Exporter DynaLoader);
 # Items to export into callers namespace by default. Note: do not export
@@ -260,10 +260,21 @@ sub uncompress($)
 }
 
 
-sub MAGIC1() { 0x1f }
-sub MAGIC2() { 0x8b }
-sub OSCODE() { 3    }
+# Constants
+use constant MAGIC1	=> 0x1f ;
+use constant MAGIC2	=> 0x8b ;
+use constant OSCODE	=> 3 ;
 
+use constant FTEXT	=> 1 ;
+use constant FHCRC	=> 2 ;
+use constant FEXTRA	=> 4 ;
+use constant FNAME	=> 8 ;
+use constant FCOMMENT	=> 16 ;
+use constant NULL	=> pack("C", 0) ;
+use constant RESERVED	=> 0xE0 ;
+
+use constant MIN_HDR_SIZE => 10 ; # minimum gzip header size
+ 
 sub memGzip
 {
   my $x = deflateInit(
@@ -274,7 +285,8 @@ sub memGzip
  
   # write a minimal gzip header
   my(@m);
-  push @m, pack("c10", MAGIC1, MAGIC2, Z_DEFLATED(), 0,0,0,0,0,0, OSCODE) ;
+  push @m, pack("c" . MIN_HDR_SIZE, 
+                MAGIC1, MAGIC2, Z_DEFLATED(), 0,0,0,0,0,0, OSCODE) ;
  
   # if the deflation buffer isn't a reference, make it one
   my $string = (ref $_[0] ? $_[0] : \$_[0]) ;
@@ -294,14 +306,91 @@ sub memGzip
   return join "", @m;
 }
 
+sub _removeGzipHeader
+{
+    my $string = shift ;
+
+    return Z_DATA_ERROR() 
+        if length($$string) < MIN_HDR_SIZE ;
+
+    my ($magic1, $magic2, $method, $flags, $time, $xflags, $oscode) = 
+        unpack ('CCCCVCC', $$string);
+
+    return Z_ERR()
+        unless $magic1 == MAGIC1 and $magic2 == MAGIC2 and
+           $method == Z_DEFLATED() and !($flags & RESERVED()) ;
+    substr($$string, 0, MIN_HDR_SIZE) = '' ;
+
+    # skip extra field
+    if ($flags & FEXTRA)
+    {
+        return Z_DATA_ERROR()
+            if length($$string) < 2 ;
+
+        my ($extra_len) = unpack ('v', $$string);
+        $extra_len += 2;
+        return Z_DATA_ERROR()
+            if length($$string) < $extra_len ;
+
+        substr($$string, 0, $extra_len) = '';
+    }
+
+    # skip orig name
+    if ($flags & FNAME)
+    {
+        my $name_end = index ($$string, NULL);
+        return Z_DATA_ERROR()
+           if $name_end == -1 ;
+        substr($$string, 0, $name_end + 1) =  '';
+    }
+
+    # skip comment
+    if ($flags & FCOMMENT)
+    {
+        my $comment_end = index ($$string, NULL);
+        return Z_DATA_ERROR()
+            if $comment_end == -1 ;
+        substr($$string, 0, $comment_end + 1) = '';
+    }
+
+    # skip header crc
+    if ($flags & FHCRC)
+    {
+        return Z_DATA_ERROR()
+            if length ($$string) < 2 ;
+        substr($$string, 0, 2) = '';
+    }
+    
+    return Z_OK();
+}
+
+
+sub memGunzip
+{
+    # if the buffer isn't a reference, make it one
+    my $string = (ref $_[0] ? $_[0] : \$_[0]);
+ 
+    _removeGzipHeader($string) == Z_OK() 
+        or return undef;
+     
+    my $x = inflateInit( -WindowBits => - MAX_WBITS()) 
+              or return undef;
+    my ($output, $status) = $x->inflate($string);
+    return undef 
+        unless $status == Z_STREAM_END();
+    my ($crc, $len) = unpack ("VV", substr($$string, 0, 8));
+    substr($$string, 0, 8) = '';
+    return undef 
+        unless $len == length($output) and
+               $crc == crc32($output);
+    return $output;   
+}
 
-1 ;
 # Autoload methods go after __END__, and are processed by the autosplit program.
 
 1;
 __END__
 
-
 =cut
 
 =head1 NAME
@@ -334,6 +423,7 @@ Compress::Zlib - Interface to zlib compression library
     $gzerrno
 
     $dest = Compress::Zlib::memGzip($buffer) ;
+    $dest = Compress::Zlib::memGunzip($buffer) ;
 
     $crc = adler32($buffer [,$crc]) ;
     $crc = crc32($buffer [,$crc]) ;
@@ -343,7 +433,7 @@ Compress::Zlib - Interface to zlib compression library
 =head1 DESCRIPTION
 
 The I<Compress::Zlib> module provides a Perl interface to the I<zlib>
-compression library (see L</AUTHORS> for details about where to get
+compression library (see L</AUTHOR> for details about where to get
 I<zlib>). Most of the functionality provided by I<zlib> is available
 in I<Compress::Zlib>.
 
@@ -382,7 +472,7 @@ The function optionally takes a number of named options specified as
 C<-Name=E<gt>value> pairs. This allows individual options to be
 tailored without having to specify them all in the parameter list.
 
-For backward compatability, it is also possible to pass the parameters
+For backward compatibility, it is also possible to pass the parameters
 as a reference to a hash containing the name=>value pairs.
 
 The function takes one optional parameter, a reference to a hash.  The
@@ -429,7 +519,7 @@ The default is C<-Strategy =E<gt>Z_DEFAULT_STRATEGY>.
 
 When a dictionary is specified I<Compress::Zlib> will automatically
 call B<deflateSetDictionary> directly after calling B<deflateInit>. The
-Adler32 value for the dictionary can be obtained by calling tht method 
+Adler32 value for the dictionary can be obtained by calling the method 
 C<$d->dict_adler()>.
 
 The default is no dictionary.
@@ -471,7 +561,7 @@ case that any output will be produced by this method. So don't rely on
 the fact that B<$out> is empty for an error test.
 
 
-=head2 B<($out, $status) = $d-E<gt>flush()>
+=head2 B<($out, $status) = $d-E<gt>flush([flush_type])>
 
 Finishes the deflation. Any pending output will be returned via B<$out>.
 B<$status> will have a value C<Z_OK> if successful.
@@ -481,6 +571,12 @@ In a scalar context B<flush> will return B<$out> only.
 Note that flushing can degrade the compression ratio, so it should only
 be used to terminate a decompression.
 
+By default the C<flush_type> used is C<Z_FINISH>. Other valid values
+for C<flush_type> are Z_NO_FLUSH, Z_PARTIAL_FLUSH, Z_SYNC_FLUSH
+and Z_FULL_FLUSH. It is strongly recommended that you only set the
+C<flush_type> parameter if you fully understand what it does. See the
+C<zlib> documentation for details.
+
 =head2 B<$d-E<gt>dict_adler()>
 
 Returns the adler32 value for the dictionary.
@@ -540,7 +636,7 @@ The function optionally takes a number of named options specified as
 C<-Name=E<gt>value> pairs. This allows individual options to be
 tailored without having to specify them all in the parameter list.
  
-For backward compatability, it is also possible to pass the parameters
+For backward compatibility, it is also possible to pass the parameters
 as a reference to a hash containing the name=>value pairs.
  
 The function takes one optional parameter, a reference to a hash.  The
@@ -829,6 +925,18 @@ undef.
 
 The buffer parameter can either be a scalar or a scalar reference.
 
+=head2 Compress::Zlib::memGunzip
+
+This function is used to uncompress an in-memory gzip file.
+
+    $dest = Compress::Zlib::memGunzip($buffer) ;
+
+If successful, it returns the uncompressed gzip file, otherwise it
+returns undef.
+
+The buffer parameter can either be a scalar or a scalar reference. The contents
+of the buffer parameter are destroyed after calling this function.
+
 =head1 CHECKSUM FUNCTIONS
 
 Two functions are provided by I<zlib> to calculate a checksum. For the
@@ -845,8 +953,14 @@ If the $crc parameters is C<undef>, the crc value will be reset.
 
 =head1 ACCESSING ZIP FILES
 
-If you want to use this module to access zip files there are a number of
-undocumented features in the zlib library you need to be aware of.
+Although it is possible to use this module to access .zip files, there
+is a module on CPAN that will do all the hard work for you. Check out
+
+    http://www.cpan.org/modules/by-module/Archive/Archive-Zip-*.tar.gz    
+
+Assuming you don't want to use this module to access zip files there
+are a number of undocumented features in the zlib library you need to
+be aware of.
 
 =over 5
 
@@ -859,7 +973,7 @@ header.
 =item 2.
 
 The zlib function B<inflate>, and so the B<inflate> method supplied in
-this module, assume that there is at leat one trailing byte after the
+this module, assume that there is at least one trailing byte after the
 compressed data stream. Normally this isn't a problem because both
 the gzip and zip file formats will guarantee that there is data directly
 after the compressed data stream.
@@ -878,240 +992,8 @@ F<Paul.Marquess@btinternet.com>. The latest copy of the module can be
 found on CPAN in F<modules/by-module/Compress/Compress-Zlib-x.x.tar.gz>.
 
 The primary site for the I<zlib> compression library is
-F<http://www.cdrom.com/pub/infozip/zlib/>.
+F<http://www.info-zip.org/pub/infozip/zlib/>.
 
 =head1 MODIFICATION HISTORY
 
-=head2 0.1 2nd October 1995.
-
-First public release of I<Compress::Zlib>.
-
-
-=head2 0.2 5th October 1995.
-
-Fixed a minor allocation problem in Zlib.xs
-
-
-=head2 0.3 12th October 1995.
-
-Added prototype specification.
-
-
-=head2 0.4 25th June 1996.
-
-=over 5
-
-=item 1.
-
-Documentation update.
-
-=item 2.
-
-Upgraded to support zlib 1.0.2
-
-=item 3.
-
-Added dictionary interface.
-
-=item 4.
-
-Fixed bug in gzreadline - previously it would keep returning the same
-buffer. This bug was reported by Helmut Jarausch
-
-=item 5.
-
-Removed dependancy to zutil.h and so dropped support for 
-	
-    DEF_MEM_LEVEL (use MAX_MEM_LEVEL instead)
-    DEF_WBITS     (use MAX_WBITS instead)
-
-=back
-
-=head2 0.50 19th Feb 1997
-
-=over 5
-
-=item 1.
-
-Confirmed that no changes were necessary for zlib 1.0.3 or 1.0.4.
-
-=item 2.
-
-The optional parameters for deflateInit and inflateInit can now be
-specified as an associative array in addition to a reference to an
-associative array. They can also accept the -Name syntax.
-
-=item 3.
-
-gzopen can now optionally take a reference to an open filehandle in
-place of a filename. In this case it will call gzdopen.
-
-=item 4.
-
-Added gzstream example script.
-
-=back
-
-=head2 1.00 14 Nov 1997
-
-=over 5
-
-=item 1.
-
-The following functions can now take a scalar reference in place of a
-scalar for their buffer parameters:
-
-    compress
-    uncompress
-    deflate
-    inflate
-    crc32
-    adler32
-
-This should mean applications that make use of the module don't have to
-copy large buffers around.
-
-=item 2.
-
-Normally the inflate method consumes I<all> of the input buffer before
-returning. The exception to this is when inflate detects the end of the
-stream (Z_STREAM_END). In this case the input buffer need not be
-completely consumed. To allow processing of file formats that embed a
-deflation stream (e.g. zip, gzip), the inflate method now sets the
-buffer parameter to be what remains after inflation.
-
-When the return status is Z_STREAM_END, it will be what remains of the
-buffer (if any) after deflation. When the status is Z_OK it will be an
-empty string.
-
-This change means that the buffer parameter must be a lvalue.
-
-=item 3.
-
-Fixed crc32 and adler32. They were both very broken. 
-
-=item 4,
-
-Added the Compress::Zlib::memGzip function.
-
-=back
-
-=head2 1.01 23 Nov 1997
-
-=over 5
-
-=item 1.
-
-A number of fixes to the test suite and the example scripts to allow
-them to work under win32. All courtesy of Gurusamy Sarathy.
-
-=back
-
-=head2 1.02 10 Jan 1998
-
-=over 5
-
-=item 1.
-
-The return codes for gzread, gzreadline and gzwrite were documented
-incorrectly as returning a status code.
-
-=item 2.
-
-The test harness was missing a "gzclose". This caused problem showed up
-on an amiga. Thanks to Erik van Roode for reporting this one.
-
-=item 3.
-
-Patched zlib.t for OS/2. Thanks to Ilya Zakharevich for the patch.
-
-=back
-
-=head2 1.03 17 Mar 1999
-
-=over 5
-
-=item 1.
-
-Updated to use the new PL_ symbols. 
-Means the module can be built with Perl 5.005_5*
-
-=head2 1.04 27 May 1999
-
-=over 5
-
-=item 1.
-
-Bug 19990527.001: compress(undef) core dumps -- Fixed.
-
-=back
-
-=head2 1.05 3 June 1999
-
-=over 5
-
-=item 1.
-
-Previous release used newSVpvn, which doesn't exist in 5.004_04 or
-earlier. Changed to use newSVpv instead.
-
-=item 2.
-
-The module needs Perl 5.004 or better, so updated the version checking in
-Zlib.pm and Makefile.PL
-
-=back
-
-=head2 1.06 20 September 1999
-
-=over 5
-
-=item 1.
-
-Fixed a nasty problem where inflate could truncate the data
-returned. Thanks to Douglas Thomson <dougt@mugc.cc.monash.edu.au> for
-both spotting the problem and fixing the bug.
-
-=item 2.
-
-Added a note about the undocumented features in zlib that are required
-when accessing zip files.
-
-=item 3.
-
-gzclose will now get called automaticallly when the gzip object is
-destroyed.
-
-=back
-
-=head2 1.07 27 November 1999
-
-=over 5
-
-=item 1.
-
-ANSI-ified the static functions in Zlib.xs
-
-=item 2.
-
-Added the ability to build zlib along with the module.
-This feature is 90% based on a Makefile provided by Gurusamy Sarathy.
-
-=back
-
-=head2 1.08 27 January 2000
-
-=over 5
-
-=item 1.
-
-uncompress was clobbering its input parameter. Now it doesn't.  
-This bug was spotted by Deven T. Corzine.
-
-=item 2.
-
-If a variable that only ever contained a number was given to compress
-or delate, it would not be compresed properly. Now it will be coerced
-to a string and then compressed. This bug was spotted by Deven T. Corzine.
-
-=back
+See the README file.
